const webdriver = require('webdriverio');
const serverPromise = require('../../test-kit/test-server');

const _ = require('lodash');

const { chai } = require('../../test-kit/test-helpers');
const { waitForServerUp } = require('../../test-kit/e2e-aux');

const { expect } = chai;

const TIME_TO_WAIT_FOR_VISIBILITY = 15000;

const options = {
  desiredCapabilities: {
    browserName: 'chrome'
  }
};

function startApplication() {
  return waitForServerUp(serverPromise)
    .then(serverDetails => {
      const serverInfo = serverDetails.serverInfo;
      if (process.env.SELENIUM_HOST) {
        options.host = process.env.SELENIUM_HOST;
      }
      if (process.env.SELENIUM_PORT) {
        options.port = process.env.SELENIUM_PORT;
      }
      const appHost = process.env.INCOMING_TEST_HOST || serverInfo.address;
      const appPort = process.env.INCOMING_TEST_PORT || serverInfo.port;
      return {client: webdriver.remote(options)
              .init()
              .setViewportSize({
                width: 1400, height: 1000
              })
              .url(`http://${appHost}:${appPort}`)};
    });
}

describe('Core3 Stack Visualizer', function () {
  this.timeout(40000);

  function getLiabilitiesTexts(client) {
    const idxArray = _.times(5);
    return idxArray.map(idx => {
      const selectRowByIndex = `[data-automation-id~="package-row-${idx}"]`;
      const arrayOfSelectors = [`${selectRowByIndex} [data-automation-id~="package-row-header-name"]`]
        .concat(idxArray.map(idx => `${selectRowByIndex} [data-automation-id~="package-row-child-${idx}"]`));

      return Promise.all(arrayOfSelectors.map(selector => client.getText(selector).catch(() => Promise.resolve(''))))
        .then(textsQueryResults => {
        const [ header, ...liabilities ] = textsQueryResults;
        return { header, liabilities: liabilities.filter(liab => liab !== '') };
      });
    });
  }

  function getLiabilitiesYs(client) {
    const idxArray = _.times(5);
    return idxArray.map(idx => {
      const selectRowByIndex = `[data-automation-id~="package-row-${idx}"]`;
      const arrayOfSelectors = [`${selectRowByIndex} [data-automation-id~="package-row-header"]`]
        .concat(idxArray.map(idx => `${selectRowByIndex} [data-automation-id~="package-row-child-${idx}"]`));

      return Promise.all(arrayOfSelectors.map(selector => client.getLocation(selector, 'y').catch(() => Promise.resolve('-1'))))
        .then(results => results.filter(number => number > 0));
    });
  }

  it('Should start application', function () {
    return startApplication()
      .then(({client}) => {
        return client.waitForVisible('[data-automation-id~="package-table"]', TIME_TO_WAIT_FOR_VISIBILITY)
          .then(() => Promise.all(getLiabilitiesTexts(client)))
          .then(liabilitiesViewRows => {
            expect(liabilitiesViewRows[0]).to.havePackage('test-p-1').and.haveLiabilities('test-p-1-1 test-p-1-2');
            expect(liabilitiesViewRows[1]).to.havePackage('test-p-1-2').and.haveLiabilities('test-p-1-2-1 test-p-1-2-2');
            expect(liabilitiesViewRows[2]).to.havePackage('test-p-1-1').and.haveLiabilities('');
            expect(liabilitiesViewRows[3]).to.havePackage('test-p-1-2-1').and.haveLiabilities('');
            expect(liabilitiesViewRows[4]).to.havePackage('test-p-1-2-2').and.haveLiabilities('');
          })
          .then(() => Promise.all(getLiabilitiesYs(client)))
          .then(liabilitiesRowHeights => {
            for (let i = 0; i < liabilitiesRowHeights.length; i++)
            {
              expect(liabilitiesRowHeights[i]).to.haveSameYCoord;
            }
            client.end();
          });
      });
  });
});
