const chai = require('chai');
const expect = chai.expect;
const request = require('request');
const stackFile = require('../../test-kit/fixtures/stack.test.json');
const path = require('path');
process.env.STACK_VISUALIZER_CONFIG_PATH = path.join(__dirname, '../../test-kit/config.test.js');
const server = require('../../src/server/server');
delete process.env.STACK_VISUALIZER_CONFIG_PATH;

const config = require('../../test-kit/config.test');


function getFromServer(fileUrl) {
  return new Promise(function (resolve, reject) {
    request.get({url: fileUrl, json: true}, function (error ,r, data) {
      if (error) {
        reject(error);
      }
      resolve({statusCode: r.statusCode, data});
    });
  });
}

let serverAddress;

function waitForServer(serverPromise) {
  return serverPromise
    .then(server => {
      const serverInfo = server.address();
      serverAddress = {
        address: serverInfo.address === '::' ? 'localhost' : serverInfo.address,
        port: serverInfo.port
      };
    })
    .catch(() => {
      throw new Error('Could not start server');
    });
}

describe('Getting files from server', function () {
  it('should provide the correct file upon request', function () {
    const serverPromise = server.createServer({get: mockGoodRequest});
    return waitForServer(serverPromise)
      .then(() => {
        return getFromServer(`http://${serverAddress.address}:${serverAddress.port}${config.apiStackFileUrl}`)
          .then(function (res) {
            expect(res.data).to.deep.equal(stackFile);
            return Promise.resolve();
          });
      });
  });
});

describe('Server error handling', function () {

  it('should provide the correct message if there is an error fetching the file', function () {
    const serverPromise = server.createServer({get: mockBadRequest});
    return waitForServer(serverPromise)
      .then(() => {
        return getFromServer(`http://${serverAddress.address}:${serverAddress.port}${config.apiStackFileUrl}`)
          .then((res) => {
            expect(res.statusCode).to.equal(500);
            expect(res.data).to.contain('Error fetching files from Github: ');
            return Promise.resolve();
          });
      })
  });

  it('should provide the correct error message if the server cannot connect to github', function () {
    const serverPromise = server.createServer({get: mockRequestWithNoConnection});
    return waitForServer(serverPromise)
      .then(() => {
        return getFromServer(`http://${serverAddress.address}:${serverAddress.port}${config.apiStackFileUrl}`)
          .then(function (res) {
            expect(res.statusCode).to.equal(500);
            expect(res.data).to.contain('Unable to reach Github server. error: ');
            return Promise.resolve();
          });
      });
  })
});



function createGithubError(){
  return {
    "message":"Not Found","documentation_url":"https://developer.github.com/v3"
  }
}

function createFakeResponse(data, statusCode = 200){
  function on(event, cb) {
    if (event === 'data') {
      cb(data);
    }

    if (event === 'end') {
      cb();
    }
  }

  return {
    statusCode,
    on
  }
}


function mockBadRequest(options, cb) {
  cb(createFakeResponse(createGithubError(), 500));
  return {on: ()=>{}};
}

function mockGoodRequest(options, cb) {
  cb(createFakeResponse(JSON.stringify(stackFile), 200));
  return {on: ()=>{}};
}

function mockRequestWithNoConnection(options, cb) {
  return {
    on: function (event, errCb) {
      if (event === 'error') {
        return errCb('error');
      }
    }
  };
}