const chai = require('chai');
chai.use(require('chai-dom'));
const expect = chai.expect;
import { PackageTable } from '../../../src/client/package-table/package-table';
import * as automationIds from '../../../src/client/common/automation-ids';
import { noPackages, packageWithoutLiabilities, packageWithLiabilities, packagesWithLiabilities } from './fixtures';

describe('The Packages Table', function () {
  let packageTableInstance = null;
  let packageTableDom = null;

  function element (automationId) {
    return packageTableDom.querySelectorAll('[data-automation-id~="' + automationId + '"]');
  }

  describe('The packages table with no packages', function () {
    beforeEach(function () {
      packageTableInstance = new PackageTable(noPackages);
      packageTableDom = packageTableInstance.getDomTable();
    })

    it('should have the text "No Results"', function () {
      expect(element(automationIds.noPackages)).to.have.text('No Results');
    });
  });

  describe('The packages table with one package that has no liabilities', function () {
    beforeEach(function () {
      packageTableInstance = new PackageTable(packageWithoutLiabilities);
      packageTableDom = packageTableInstance.getDomTable();
    });

    it('should display the package', function () {
      expect(element(automationIds.packageName)).to.have.text('test-p-1');
    });

    describe('The package', function () {
      it('should show the package version', function () {
        expect(element(automationIds.packageVersion)).to.have.text('Version: 1.0.0');
      });
    });
  });

  describe('The packages table with one package that has liabilities', function () {
    beforeEach(function () {
      packageTableInstance = new PackageTable(packageWithLiabilities);
      packageTableDom = packageTableInstance.getDomTable();
    });

    it('should display the package', function () {
      expect(element(automationIds.packageName)).to.have.text('test-p-1');
    });

    describe('The package', function () {
      it('should show the package version', function () {
        expect(element(automationIds.packageVersion)).to.have.text('Version: 1.0.0');
      });
    });

    it('should show the liabilities', function () {
      expect(element(automationIds.rowElement + '-0')).to.have.text('test-p-1-1');
      expect(element(automationIds.rowElement + '-1')).to.have.text('test-p-1-2');
    });
  });

  describe('The packages table with multiple packages and multiple liabilities', function () {
    beforeEach(function () {
      packageTableInstance = new PackageTable(packagesWithLiabilities);
      packageTableDom = packageTableInstance.getDomTable();
    });

    it('should display the packages', function () {
      expect(element(automationIds.packageName)[0]).to.have.text('test-p-1');
      expect(element(automationIds.packageName)[1]).to.have.text('test-p-1-1');
      expect(element(automationIds.packageName)[2]).to.have.text('test-p-1-2');
      expect(element(automationIds.packageName)[3]).to.have.text('test-p-1-2-2');
      expect(element(automationIds.packageName)[4]).to.have.text('test-p-1-2-1');
    });

    describe('The packages', function () {
      it('should show the package version', function () {
        expect(element(automationIds.packageVersion)[0]).to.have.text('Version: 1.0.0');
        expect(element(automationIds.packageVersion)[1]).to.have.text('Version: 1.1.0');
        expect(element(automationIds.packageVersion)[2]).to.have.text('Version: 1.0.0');
        expect(element(automationIds.packageVersion)[3]).to.have.text('Version: 1.1.0');
        expect(element(automationIds.packageVersion)[4]).to.have.text('Version: 1.89.01');
      });
    });

    it('should have 5 rows', function () {
      const rows = [];
      // 5 rows expected
      for (let i = 0; i < 5; i++) {
        rows.push(element(automationIds.row + '-' + i));
      }
      expect(rows.length).to.equal(5);

    });
  });
});